@@ -1,20 +1,8 @@
+// SPDX-License-Identifier: GPL-2.0-only
 /*
  * NXP Wireless LAN device driver: PCIE specific handling
  *
  * Copyright 2011-2020 NXP
- *
- * This software file (the "File") is distributed by NXP
- * under the terms of the GNU General Public License Version 2, June 1991
- * (the "License").  You may use, redistribute and/or modify this File in
- * accordance with the terms and conditions of the License, a copy of which
- * is available by writing to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA or on the
- * worldwide web at http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
- *
- * THE FILE IS DISTRIBUTED AS-IS, WITHOUT WARRANTY OF ANY KIND, AND THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE
- * ARE EXPRESSLY DISCLAIMED.  The License provides additional details about
- * this warranty disclaimer.
  */
 
 #include <linux/iopoll.h>
@@ -574,24 +562,10 @@ static void mwifiex_pcie_reset_done(struct pci_dev *pdev)
 
 	card->pci_reset_ongoing = false;
 }
-#if LINUX_VERSION_IS_LESS(4,13,0)
-static void mwifiex_pcie_reset_prepare_notify(struct pci_dev *dev,
-					      bool prepare){
-	if (prepare)
-		mwifiex_pcie_reset_prepare(dev);
-	else
-		mwifiex_pcie_reset_done(dev);
-}
-#endif
 
 static const struct pci_error_handlers mwifiex_pcie_err_handler = {
-#if LINUX_VERSION_IS_GEQ(4,13,0)
 	.reset_prepare		= mwifiex_pcie_reset_prepare,
 	.reset_done		= mwifiex_pcie_reset_done,
-#else
-	.reset_notify = mwifiex_pcie_reset_prepare_notify,
-#endif
-
 };
 
 #ifdef CONFIG_PM_SLEEP
@@ -676,7 +650,7 @@ static int mwifiex_pm_wakeup_card(struct mwifiex_adapter *adapter)
 {
 	struct pcie_service_card *card = adapter->card;
 	const struct mwifiex_pcie_card_reg *reg = card->pcie.reg;
-	int retval;
+	int retval __maybe_unused;
 
 	mwifiex_dbg(adapter, EVENT,
 		    "event: Wakeup device...\n");
@@ -3400,7 +3374,7 @@ static void mwifiex_unregister_dev(struct mwifiex_adapter *adapter)
 	} else {
 		mwifiex_dbg(adapter, INFO,
 			    "%s(): calling free_irq()\n", __func__);
-	       free_irq(card->dev->irq, &card->share_irq_ctx);
+		free_irq(card->dev->irq, &card->share_irq_ctx);
 
 		if (card->msi_enable)
 			pci_disable_msi(pdev);
